@@ -1,4 +1,5 @@
 use crate::algorithm::{BreakToken, Printer};
+use crate::attr;
 use crate::iter::IterDelimited;
 use crate::stmt;
 use crate::INDENT;
@@ -539,7 +540,9 @@ impl Printer {
     fn expr_struct(&mut self, expr: &ExprStruct) {
         self.outer_attrs(&expr.attrs);
         self.cbox(INDENT);
+        self.ibox(-INDENT);
         self.path(&expr.path);
+        self.end();
         self.word(" {");
         self.space_if_nonempty();
         self.inner_attrs(&expr.attrs);
@@ -781,38 +784,48 @@ impl Printer {
     fn call_args(&mut self, args: &Punctuated<Expr, Token![,]>) {
         self.word("(");
         let mut iter = args.iter();
-        if let (Some(expr @ (Expr::Closure(_) | Expr::Struct(_))), None) =
-            (iter.next(), iter.next())
-        {
-            self.expr(expr);
-        } else {
-            self.cbox(INDENT);
-            self.zerobreak();
-            for arg in args.iter().delimited() {
-                self.expr(&arg);
-                self.trailing_comma(arg.is_last);
+        match (iter.next(), iter.next()) {
+            (
+                Some(
+                    expr @ (Expr::Closure(ExprClosure { attrs, .. })
+                    | Expr::Struct(ExprStruct { attrs, .. })
+                    | Expr::Block(ExprBlock { attrs, .. })),
+                ),
+                None,
+            ) if !attr::has_outer(attrs) => {
+                self.expr(expr);
+            }
+            _ => {
+                self.cbox(INDENT);
+                self.zerobreak();
+                for arg in args.iter().delimited() {
+                    self.expr(&arg);
+                    self.trailing_comma(arg.is_last);
+                }
+                self.offset(-INDENT);
+                self.end();
             }
-            self.offset(-INDENT);
-            self.end();
         }
         self.word(")");
     }
 
     fn small_block(&mut self, block: &Block, attrs: &[Attribute]) {
         self.word("{");
-        self.space_if_nonempty();
-        self.inner_attrs(attrs);
-        if let (Some(Stmt::Expr(expr)), None) = (block.stmts.get(0), block.stmts.get(1)) {
-            self.ibox(0);
-            self.expr(expr);
-            self.end();
+        if attr::has_inner(attrs) || !block.stmts.is_empty() {
             self.space();
-        } else {
-            for stmt in &block.stmts {
-                self.stmt(stmt);
+            self.inner_attrs(attrs);
+            if let (Some(Stmt::Expr(expr)), None) = (block.stmts.get(0), block.stmts.get(1)) {
+                self.ibox(0);
+                self.expr(expr);
+                self.end();
+                self.space();
+            } else {
+                for stmt in &block.stmts {
+                    self.stmt(stmt);
+                }
             }
+            self.offset(-INDENT);
         }
-        self.offset(-INDENT);
         self.word("}");
     }
 
